# -*- coding: utf-8 -*-

__author__ = 'Fabio Caccamo'
__copyright__ = 'Copyright (c) 2019-2020 Fabio Caccamo'
__description__ = 'python-benedict is a dict subclass with keylist/keypath support, I/O shortcuts (base64, csv, json, pickle, plist, query-string, toml, xml, yaml) and many utilities... for humans, obviously.'
__email__ = 'fabio.caccamo@gmail.com'
__license__ = 'MIT'
__title__ = 'benedict'
__version__ = '0.21.0'
